@@ -150,18 +150,17 @@ G_DEFINE_TYPE (PolkitBackendJsAuthority, polkit_backend_js_authority, POLKIT_BAC
 /* ---------------------------------------------------------------------------------------------------- */
 
 static const struct JSClassOps js_global_class_ops = {
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL
+  nullptr,  // addProperty
+  nullptr,  // deleteProperty
+  nullptr,  // enumerate
+  nullptr,  // newEnumerate
+  nullptr,  // resolve
+  nullptr,  // mayResolve
+  nullptr,  // finalize
+  nullptr,  // call
+  nullptr,  // hasInstance
+  nullptr,  // construct
+  JS_GlobalObjectTraceHook
 };
 
 static JSClass js_global_class = {
@@ -172,18 +171,17 @@ static JSClass js_global_class = {
 
 /* ---------------------------------------------------------------------------------------------------- */
 static const struct JSClassOps js_polkit_class_ops = {
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL
+  nullptr,  // addProperty
+  nullptr,  // deleteProperty
+  nullptr,  // enumerate
+  nullptr,  // newEnumerate
+  nullptr,  // resolve
+  nullptr,  // mayResolve
+  nullptr,  // finalize
+  nullptr,  // call
+  nullptr,  // hasInstance
+  nullptr,  // construct
+  nullptr   // trace
 };
 
 static JSClass js_polkit_class = {
@@ -469,19 +467,18 @@ polkit_backend_js_authority_constructed (GObject *object)
 
   {
     JS::CompartmentOptions compart_opts;
-    compart_opts.behaviors().setVersion(JSVERSION_LATEST);
+
     JS::RootedObject global(authority->priv->cx);
 
     authority->priv->js_global = new JS::Heap<JSObject*> (JS_NewGlobalObject (authority->priv->cx, &js_global_class, NULL, JS::FireOnNewGlobalHook, compart_opts));
 
     global = authority->priv->js_global->get ();
-
-    if (global == NULL)
+    if (!global)
       goto fail;
 
     authority->priv->ac = new JSAutoCompartment(authority->priv->cx,  global);
 
-    if (authority->priv->ac == NULL)
+    if (!authority->priv->ac)
       goto fail;
 
     if (!JS_InitStandardClasses (authority->priv->cx, global))
@@ -493,7 +490,7 @@ polkit_backend_js_authority_constructed (GObject *object)
 
     polkit = authority->priv->js_polkit->get ();
 
-    if (polkit == NULL)
+    if (!polkit)
       goto fail;
 
     if (!JS_DefineProperty(authority->priv->cx, global, "polkit", polkit, JSPROP_ENUMERATE))
@@ -504,7 +501,7 @@ polkit_backend_js_authority_constructed (GObject *object)
                              js_polkit_functions))
       goto fail;
 
-    JS::CompileOptions options(authority->priv->cx, JSVERSION_UNKNOWN);
+    JS::CompileOptions options(authority->priv->cx);
     JS::RootedValue rval(authority->priv->cx);
     if (!JS::Evaluate (authority->priv->cx,
                        options,
@@ -684,7 +681,9 @@ set_property_strv (PolkitBackendJsAuthority  *authority,
   JS::AutoValueVector elems(authority->priv->cx);
   guint n;
 
-  elems.resize(value->len);
+  if (!elems.resize(value->len))
+    g_error ("Unable to resize vector");
+
   for (n = 0; n < value->len; n++)
     {
       const char *c_string = (const char *) g_ptr_array_index(value, n);
@@ -741,7 +740,7 @@ subject_to_jsval (PolkitBackendJsAuthority  *authority,
                   GError                   **error)
 {
   gboolean ret = FALSE;
-  JS::CompileOptions options(authority->priv->cx, JSVERSION_UNKNOWN);
+  JS::CompileOptions options(authority->priv->cx);
   const char *src;
   JS::RootedObject obj(authority->priv->cx);
   pid_t pid;
@@ -868,7 +867,7 @@ action_and_details_to_jsval (PolkitBackendJsAuthority  *authority,
                              GError                   **error)
 {
   gboolean ret = FALSE;
-  JS::CompileOptions options(authority->priv->cx, JSVERSION_UNKNOWN);
+  JS::CompileOptions options(authority->priv->cx);
   const char *src;
   JS::RootedObject obj(authority->priv->cx);
   gchar **keys;
@@ -1285,14 +1284,16 @@ js_polkit_log (JSContext  *cx,
                unsigned    argc,
                JS::Value      *vp)
 {
-  /* PolkitBackendJsAuthority *authority = POLKIT_BACKEND_JS_AUTHORITY (JS_GetContextPrivate (cx)); */
+  PolkitBackendJsAuthority *authority = POLKIT_BACKEND_JS_AUTHORITY (JS_GetContextPrivate (cx));
   bool ret = false;
   char *s;
 
   JS::CallArgs args = JS::CallArgsFromVp (argc, vp);
 
-  s = JS_EncodeString (cx, args[0].toString ());
-  JS_ReportWarningUTF8 (cx, s);
+  JS::RootedString jsstr (authority->priv->cx);
+  jsstr = args[0].toString ();
+  s = JS_EncodeStringToUTF8 (cx, jsstr);
+  JS_ReportWarningUTF8 (cx, "%s", s);
   JS_free (cx, s);
 
   ret = true;
@@ -1368,7 +1369,7 @@ js_polkit_spawn (JSContext  *cx,
                  unsigned    js_argc,
                  JS::Value      *vp)
 {
-  /* PolkitBackendJsAuthority *authority = POLKIT_BACKEND_JS_AUTHORITY (JS_GetContextPrivate (cx)); */
+  PolkitBackendJsAuthority *authority = POLKIT_BACKEND_JS_AUTHORITY (JS_GetContextPrivate (cx));
   bool ret = false;
   JS::RootedObject array_object(cx);
   gchar *standard_output = NULL;
@@ -1408,7 +1409,9 @@ js_polkit_spawn (JSContext  *cx,
           JS_ReportErrorUTF8 (cx, "Element %d is not a string", n);
           goto out;
 	}
-      s = JS_EncodeString (cx, elem_val.toString());
+      JS::RootedString jsstr (authority->priv->cx);
+      jsstr = elem_val.toString();
+      s = JS_EncodeStringToUTF8 (cx, jsstr);
       argv[n] = g_strdup (s);
       JS_free (cx, s);
     }
@@ -1491,7 +1494,7 @@ js_polkit_user_is_in_netgroup (JSContext  *cx,
                                unsigned    argc,
                                JS::Value      *vp)
 {
-  /* PolkitBackendJsAuthority *authority = POLKIT_BACKEND_JS_AUTHORITY (JS_GetContextPrivate (cx)); */
+  PolkitBackendJsAuthority *authority = POLKIT_BACKEND_JS_AUTHORITY (JS_GetContextPrivate (cx));
   bool ret = false;
   char *user;
   char *netgroup;
@@ -1499,8 +1502,12 @@ js_polkit_user_is_in_netgroup (JSContext  *cx,
 
   JS::CallArgs args = JS::CallArgsFromVp (argc, vp);
 
-  user = JS_EncodeString (cx, args[0].toString());
-  netgroup = JS_EncodeString (cx, args[1].toString());
+  JS::RootedString usrstr (authority->priv->cx);
+  usrstr = args[0].toString();
+  user = JS_EncodeStringToUTF8 (cx, usrstr);
+  JS::RootedString netgstr (authority->priv->cx);
+  netgstr = args[1].toString();
+  netgroup = JS_EncodeStringToUTF8 (cx, netgstr);
 
   if (innetgr (netgroup,
                NULL,  /* host */
@@ -1595,7 +1602,8 @@ utils_spawn_data_free (UtilsSpawnData *data)
                              (GSourceFunc) utils_child_watch_from_release_cb,
                              source,
                              (GDestroyNotify) g_source_destroy);
-      g_source_attach (source, data->main_context);
+      /* attach source to the global default main context */
+      g_source_attach (source, NULL);
       g_source_unref (source);
       data->child_pid = 0;
     }
